/*
** Module   : XEntityImpl
** Abstract : DOM XML document and nodereference commons implementation.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130204 Created initial version. Major rework to use w3c backend.
** 002 EVL 20130214 Adding the base inheritance from XCommonImpl class to place the XML related
**                  common features.  
*/

package com.goldencode.p2j.xml;

import java.util.*;
import org.w3c.dom.*;
import com.goldencode.p2j.util.*;

/**
 * Implementation of the common features for X-Docunet and X-Node-Reference
 * Progress DOM XML support objects.
 */
public class XEntityImpl
extends XCommonImpl
implements XEntity
{
   /** The handle object associated with this node or document. */
   private handle hXEntity = null;

   /** List of child node handles. */
   private Map<Element, handle> childNodeList;

   /** Element associates with current XEntity class. */
   private Element xnode = null;  
   
   /**
    * Default constructor.
    */
   public XEntityImpl()
   {
      hXEntity = new handle();
      
      childNodeList  = Collections.synchronizedMap(
                                     new LinkedHashMap<Element, handle>());
   }

   /**
    * Special constructor accepting the X-Document or X-Node-Reference object.
    * 
    * @param    value
    *           The object to be associated with the given handle object.
    */
   public XEntityImpl(WrappedResource value)
   {
      this();
      hXEntity.assign(value);
   }

   /**
    * Special constructor accepting the X-Document or X-Node-Reference object.
    * 
    * @param    element
    *           The DOM XML Element object to be associated with the given
    *           handle object.
    */
   public XEntityImpl(Element element)
   {
      this();
      xnode = element;
      hXEntity.assign(this);
   }

   /**
    * Getting the Element backend object for this XEntity.
    * 
    * @return   The Element associated with object.
    */
   public Element getNode()
   {
      return xnode;
   }
   
   /**
    * Getting the handle of the object.  
    *
    * @return   Internal handle obect associated with this node.
    */
   public handle getHandle()
   {
      return hXEntity;
   }
   
   /**
    * Reports if this object is valid for use.  
    *
    * @return   <code>true</code> if we are valid (can be used).
    */
   public boolean isValid()
   {
      return super.isValid() && hXEntity != null && xnode != null;
   }
   
   /**
    * Reports if this object is unknown.  
    *
    * @return   <code>true</code> if object is unknown.
    */
   public boolean isUnknown()
   {
      return super.isUnknown() && (hXEntity == null || xnode == null);
   }

   // Attributes
   /**
    * Gets the the number of children objects for the current one.
    * 
    * @return   The integer number of the children.
    */
   public integer getNumChildren()
   {
      return new integer(xnode.getChildNodes().getLength());
   }

   /**
    * Gets the the unique ID number associated to this object by the
    * underlying system. Not the same as the handle value itself.
    * 
    * @return   The integer number of the children.
    */
   public integer getUniqueId()
   {
      return new integer(hXEntity.hashCode());
   }

   // Methods
   /**
    * Appending the new child to the current object.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be appended.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical appendChild(handle hChild)
   {
      return appendChild(hChild, (character) null, (character) null);
   }

   /**
    * Appending the new child to the current object.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be appended.
    * @param    name
    *           The name of the child to be appended.
    * @param    type
    *           The type of the child to be appended.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical appendChild(handle hChild, character name, character type)
   {
      Element elChild = ((XEntityImpl)(hChild.get())).getNode();
      if (xnode.appendChild(elChild) != null)
      {
         childNodeList.put(elChild, hChild);
         return new logical(true);
      }
      else
      {
         return new logical(false);
      }
   }

   // Returning value placed into hChild
   /**
    * Getting the handle object of the child node with specified index.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to put
    *           the results.
    * @param    index
    *           1-based index of the child node to be retrieved.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical getChild(handle hChild, int index)
   {
      return getChild(hChild, new integer(index));
   }
   
   /**
    * Getting the handle object of the child node with specified index.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to put
    *           the results.
    * @param    index
    *           1-based index of the child node to be retrieved.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical getChild(handle hChild, integer index)
   {
      int ndx = index.getValue();
      
      NodeList nlst = xnode.getChildNodes();
      
      if (ndx < 1 || ndx > nlst.getLength())
         return new logical(false);
      
      Element elem = (Element)nlst.item(ndx - 1);
      hChild = childNodeList.get(elem);
  
      return new logical(hChild != null);
   }
   
   /**
    * Removing the specified child from the node.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be removed.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical removeChild(handle hChild)
   {
      boolean result = false;
      
      Element elChild = ((XEntityImpl)(hChild.get())).getNode();
      
      result = childNodeList.remove(elChild) != null;
      result = xnode.removeChild(elChild) != null;
      
      return new logical(result);
   }
   
   /**
    * Clean up the current node.
    * 
    */
   public void clear()
   {
      childNodeList.clear();
      hXEntity = null;
      xnode.getOwnerDocument().removeChild(xnode);
      xnode = null;
   }
}
